@@ -1153,9 +1153,6 @@ SRCINST_ERR _check_build_ready(struct _package_info* info) {
 SRCINST_ERR _action_upgrade(struct _package_info* old_info, struct _package_info* new_info) {
   int i;
 
-  /* remember to update when we close the old package */
-  old_info->onclose |= SRCINST_ONCLOSE_UPDATE;
-
   if (old_info->installed) {
     if (!_internal_uninstall(old_info)) {
       srcinst_warning(SRCINST_ERR_ACTION,
