@@ -35,9 +35,6 @@ struct _makefile_features {
 };
 
 struct _makefile_features* _detect_makefile_features(void);
-void _init_makefile_features(struct _makefile_features* mf);
-int _test_makefile_features(struct _makefile_features* mf, char* mfile);
-int _test_makefile_features_rec(struct _makefile_features* mf, char* mfile);
 void _free_makefile_features(struct _makefile_features* mf);
 
 #endif
