@@ -30,6 +30,7 @@ struct srcinst_parsed_data _keywords_preferences[SRCINST_PREFERENCES_NKEYS] = {
   { "KEEP_SOURCE", 0, "%d" },
   { "MANUAL_CONFIGURE", 0, "%d" },
   { "PREFIX", 0, "%s" },
+  { "QUIET", 0, "%d" },
   { "SRC_COMPRESS", 0, "%s" },
   { "STRIP", 0, "%d" }
 };
@@ -41,6 +42,7 @@ static void _init_keywords_preferences(struct _preferences* pref) {
   data[SRCINST_PKEY_KEEP_SOURCE].variable = &(pref->keep_source);
   data[SRCINST_PKEY_MANUAL_CONFIGURE].variable = &(pref->manual_configure);
   data[SRCINST_PKEY_PREFIX].variable = &(pref->prefix);
+  data[SRCINST_PKEY_QUIET].variable = &(pref->quiet);
   data[SRCINST_PKEY_SRC_COMPRESS].variable = &(pref->src_compress);
   data[SRCINST_PKEY_STRIP].variable = &(pref->strip);
 }
@@ -58,6 +60,7 @@ void _init_preferences(struct _preferences* p) {
   p->manual_configure = 1;
   p->keep_source = 0;
   p->install = 1;
+  p->quiet = 0;
   p->strip = 0;
   p->src_compress = SRCINST_COMP_GZ;
   p->add_directory = 0;
@@ -68,6 +71,7 @@ void _free_preferences(struct _preferences* p) {
   p->manual_configure = 0;
   p->keep_source = 0;
   p->install = 0;
+  p->quiet = 0;
   p->strip = 0;
   p->src_compress = 0;
   if (p->add_directory) { free(p->add_directory); p->add_directory = 0; }
