@@ -29,18 +29,28 @@ struct _cfg_keywords {
   char* name;			/* verbose name of the section */
   char* keyword;		/* regex of the keyword of the section */
   char* pattern;		/* regex of the option to match within sect */
-  struct srcinst_configure_options* o;
+
+  SRCINST_REGEX comp_keyword;	/* compiled regex for the keyword */
+  SRCINST_REGEX comp_pattern;	/* compiled regex for the option within sect */
+  struct srcinst_configure_options* o; /* reference to the relevant options */
 };
 
 #define SRC_MAIN_PATTERN "^  (--[[:alnum:]_-]+)(\\[?=[[:alnum:]_-]+\\]?)?[[:space:]]*([^[:space:]].*)?"
 #define SRC_ENV_PATTERN "^  ([[:alnum:]_]+)(\\[?=[[:alnum:]_-]+\\]?)?[[:space:]]*([^[:space:]].*)?"
+#define SRC_SKIP_PATTERN "^  --[[:alnum:]]+-(PACKAGE|FEATURE)"
+#define SRC_CONT_PATTERN "^[[:space:]]+([^[:space:]].*)"
 
 static struct _cfg_keywords _cfg_keywords[] = {
-  { "Optional Features", "^Optional Features", SRC_MAIN_PATTERN, 0 },
-  { "Optional Packages and X", "^(Optional Packages|X features)", SRC_MAIN_PATTERN, 0 },
-  { "Environment Variables", "^Some influential environment variables", SRC_ENV_PATTERN, 0 },
-  { "Optional Features and Packages", "^--enable and --with options recognized", SRC_MAIN_PATTERN, 0 },
-  { 0, 0, 0, 0 }
+  { "Optional Features", "^Optional Features", SRC_MAIN_PATTERN, 0, 0, 0 },
+  { "Optional Packages and X", "^(Optional Packages|X features)",
+    SRC_MAIN_PATTERN, 0, 0, 0},
+  { "Environment Variables", "^Some influential environment variables", 
+    SRC_ENV_PATTERN, 0, 0, 0 },
+  { "Optional Features and Packages", 
+    "^--enable and --with options recognized", SRC_MAIN_PATTERN, 0, 0, 0 },
+
+  { "Uncategorized options", 0, SRC_MAIN_PATTERN, 0, 0, 0 },
+  { 0, 0, 0, 0, 0, 0 }
 };
 
 static void _init_configure_options(struct srcinst_configure_options* o) {
@@ -63,6 +73,18 @@ static void _free_configure_options(struct srcinst_configure_options* o) {
   _init_configure_options(o);
 }
 
+static int _search_configure_options(struct srcinst_configure_options* o,
+				     char *key) {
+  int i;
+  if (o->options) {
+    for (i = 0; i < o->count; i++) {
+      if (strcmp(o->options[i].key, key) == 0)
+	return i;
+    }
+  }
+  return -1;
+}
+
 static void _init_configure_option(struct srcinst_configure_option* o, char* key, char* param, char* comment, char* value) {
   o->key = srcinst_strdup(key);
   o->param = param ? srcinst_strdup(param) : 0;
@@ -78,12 +100,10 @@ static void _add_configure_option(struct srcinst_configure_options* o, char* key
 
 static void _add_base_configure_options(struct srcinst_configure_options* o) {
   int idx; idx = o->count;
-  o->count += 13;
+  o->count += 14;
   o->options = srcinst_realloc(o->options, sizeof(struct srcinst_configure_option) * o->count);
 
-  /*
-    _init_configure_option(o->options + idx++, "--prefix", "=PREFIX", "install files in PREFIX [default=/usr/local]", 0);
-  */
+  _init_configure_option(o->options + idx++, "--prefix", "=PREFIX", "install files in PREFIX overriding default prefix", 0);
 
   _init_configure_option(o->options + idx++, "--exec-prefix", "=EPREFIX", "install architecture-dependent files in EPREFIXs [default=PREFIX]", 0);
   _init_configure_option(o->options + idx++, "--bindir", "=DIR", "user executables [EPREFIX/bin]", 0);
@@ -108,12 +128,97 @@ void _action_free_configure_opt(struct srcinst_configure_sets* sets) {
   _free_configure_options(&sets->envs);
   _free_configure_options(&sets->olds);
   _free_configure_options(&sets->res1);
-  _free_configure_options(&sets->res2);
+}
+
+static int _search_section_keyword(char *str) {
+  int i;
+
+  for (i = 0; _cfg_keywords[i].keyword; i++) {
+    if (srcinst_exec_regex(_cfg_keywords[i].comp_keyword, str, 0)) {
+      return i;
+    }
+  }
+
+  return -1;
+}
+
+static void _init_sections(struct srcinst_configure_sets *sets) {
+  struct _cfg_keywords *section;
+
+  _cfg_keywords[0].o = &sets->feats; _cfg_keywords[1].o = &sets->packs;
+  _cfg_keywords[2].o = &sets->envs; _cfg_keywords[3].o = &sets->olds;
+  _cfg_keywords[4].o = &sets->res1; _cfg_keywords[5].o = 0;
+  
+  for (section = _cfg_keywords; section->o; section++) {
+    if (section->keyword)
+      section->comp_keyword =
+	srcinst_init_regex(section->keyword, REG_EXTENDED | REG_ICASE);
+    
+    section->comp_pattern = srcinst_init_regex(section->pattern, -1);
+  }
+}
+
+static void _fini_sections(void) {
+  struct _cfg_keywords *section;
+
+  for (section = _cfg_keywords; section->o; section++) {
+    if (section->keyword)
+      srcinst_free_regex(section->comp_keyword);
+    
+    srcinst_free_regex(section->comp_pattern);
+  }
+}
+
+static int _detect_option_element(struct _cfg_keywords *section,
+				  struct srcinst_configure_options *base,
+				  SRCINST_MATCH m, 
+				  struct srcinst_string_list *output,
+				  int lineno) {
+  SRCINST_REGEX cont; SRCINST_MATCH cont_m;
+  char* key, *param, *comment;
+  cont = srcinst_init_regex(SRC_CONT_PATTERN, -1);
+  cont_m = srcinst_init_match(2);
+
+  key = srcinst_get_match(m, 1);
+  param = srcinst_get_match(m, 2);   /* can be (char*)0 */
+  comment = srcinst_get_match(m, 3); /* can be (char*)0 */
+  
+  /* scan forward to fetch continued comments */
+  
+  while ((++lineno) < output->count) {
+    char* fwline;
+    fwline = output->strings[lineno];
+    if (*fwline == '\0')
+      break;
+    if (srcinst_exec_regex(section->comp_pattern, fwline, 0))
+      break;
+    if (!srcinst_exec_regex(cont, fwline, cont_m)) break;
+    if (comment) 
+      comment = srcinst_append(comment, " ");
+    comment = srcinst_append(comment, 
+			     fwline + srcinst_get_match_offset(cont_m, 1));
+  }
+  
+  if (section->keyword || _search_configure_options(base, key) == -1)
+    _add_configure_option(section->o, key, param, comment, 0);
+
+  if (key) free(key);
+  if (param) free(param);
+  if (comment) free(comment);
+  
+  srcinst_free_match(cont_m);
+  srcinst_free_regex(cont);
+
+  return lineno - 1;
 }
 
 SRCINST_ERR _action_detect_configure_opt(struct _package_info* i, struct srcinst_configure_sets* sets) {
   struct srcinst_string_list output;
+  struct srcinst_configure_options options;
+  struct _cfg_keywords* section; 
   SRCINST_ERR rv;
+  SRCINST_REGEX skip;
+  int lineno;
   
   if ((rv = _check_build_ready(i)) != SRCINST_ERR_OK)
     return rv;
@@ -135,78 +240,39 @@ SRCINST_ERR _action_detect_configure_opt(struct _package_info* i, struct srcinst
       return SRCINST_ERR_ACTION;
 
   } else {
-    if (!srcinst_spawn_wait_expect_lines(&output, PRG_sh, "./configure", "--help", 0))
+    if (!srcinst_spawn_wait_expect_lines(&output, PRG_sh, "./configure",
+					 "--help", 0))
       return SRCINST_ERR_ACTION;
   }
 
   _add_base_configure_options(&sets->base);
-
-  _cfg_keywords[0].o = &sets->feats; _cfg_keywords[1].o = &sets->packs;
-  _cfg_keywords[2].o = &sets->envs; _cfg_keywords[3].o = &sets->olds;
-
-  do {
-    SRCINST_REGEX skip; SRCINST_REGEX cont; SRCINST_MATCH cont_m;
-    int lineno; struct _cfg_keywords* section;
-    skip = srcinst_init_regex("^  --[[:alnum:]]+-(PACKAGE|FEATURE)", -1);
-    /* printf("PASS 1\n"); fflush(stdout); */
-    cont = srcinst_init_regex("^[[:space:]]+([^[:space:]].*)", -1);
-    /* printf("PASS 2\n"); fflush(stdout); */
-    cont_m = srcinst_init_match(2);
-
-    for (lineno = 0; lineno < output.count; lineno++) {
-      for (section = _cfg_keywords; section->keyword; section++) {
-	char* line; SRCINST_REGEX r; line = output.strings[lineno];
-	r = srcinst_init_regex(section->keyword, REG_EXTENDED | REG_ICASE);
-	/* printf("PASS 3 lineno=%i\n", lineno); fflush(stdout); */
-	if (srcinst_exec_regex(r, line, 0)) {
-	  /* matched section start */
-	  SRCINST_REGEX reg; SRCINST_MATCH m;
-	  reg = srcinst_init_regex(section->pattern, -1);
-	  /* printf("PASS 4 lineno=%i\n", lineno); fflush(stdout); */
-	  m = srcinst_init_match(4);
-	  
-	  while (1) {
-	    char* key, *param, *comment; int fwidx;
-	    if (++lineno >= output.count) break;
-	    line = output.strings[lineno];
-	    if (!srcinst_exec_regex(reg, line, m))   /* sub, par, com */
-	      break;
-	    if (srcinst_exec_regex(skip, line, 0))
-	      continue;
-	    key = srcinst_get_match(m, 1);
-	    param = srcinst_get_match(m, 2);   /* can be (char*)0 */
-	    comment = srcinst_get_match(m, 3); /* can be (char*)0 */
-	    
-	    fwidx = 0;		/* scan forward to fetch continued comments */
-	    while (1) {
-	      char* fwline;
-	      ++fwidx;
-	      if (lineno + fwidx >= output.count) break;
-	      fwline = output.strings[lineno + fwidx];
-	      if (*fwline == '\0') break;
-	      if (srcinst_exec_regex(reg, fwline, 0)) break;
-	      if (!srcinst_exec_regex(cont, fwline, cont_m)) break;
-	      if (comment) comment = srcinst_append(comment, " ");
-	      comment = srcinst_append(comment, fwline + srcinst_get_match_offset(cont_m, 1));
-	    }
-	    _add_configure_option(section->o, key, param, comment, 0);
-	    if (key) free(key);
-	    if (param) free(param);
-	    if (comment) free(comment);
-	    lineno += fwidx - 1;
-	  }
-	  srcinst_free_match(m);
-	  srcinst_free_regex(reg);
-	}
-	srcinst_free_regex(r);
-	if (lineno >= output.count)
-	  break;
-      }
-    }
-    srcinst_free_regex(skip);
-    srcinst_free_regex(cont);
-    srcinst_free_match(cont_m);
-  } while (0);
+  _init_sections(sets);
   
+  skip = srcinst_init_regex(SRC_SKIP_PATTERN, -1);
+  section = _cfg_keywords + 4;	/* default section = uncategorized res1 */
+
+  for (lineno = 0; lineno < output.count; lineno++) {
+    SRCINST_MATCH m; int i;
+    i = _search_section_keyword(output.strings[lineno]);
+    if (i != -1) {
+      section = _cfg_keywords + i;
+      continue;
+    }
+    
+    m = srcinst_init_match(4);
+
+    if (srcinst_exec_regex(skip, output.strings[lineno], 0))
+      continue;
+
+    if (srcinst_exec_regex(section->comp_pattern, output.strings[lineno], m))
+      lineno = _detect_option_element(section, &sets->base, m, &output,
+				      lineno);
+    
+    srcinst_free_match(m);
+  }
+  
+  srcinst_free_regex(skip);
+  _fini_sections();
+
   return SRCINST_ERR_OK;
 }
