@@ -554,6 +554,7 @@ int srcinst_close_package(SRCINST_PHANDLE h) {
       
       if (info->upgrading_from) {
 	_action_upgrade(info->upgrading_from->info, info);
+	_unlink_package_info(info->upgrading_from->info);
 
 	_free_package_element(info->upgrading_from);
 	free(info->upgrading_from);
@@ -575,6 +576,7 @@ int srcinst_close_package(SRCINST_PHANDLE h) {
     rv = _update_package(&_srcinst_state.packages, e);
   }
 
+  info->onclose = SRCINST_ONCLOSE_NOP;
   return rv;
 }
 
@@ -780,6 +782,10 @@ char srcinst_get_pref_strip(void) {
   return _srcinst_state.preferences.strip;
 }
 
+char srcinst_get_pref_quiet(void) {
+  return _srcinst_state.preferences.quiet;
+}
+
 char srcinst_get_pref_install(void) {
   return _srcinst_state.preferences.install;
 }
@@ -824,6 +830,13 @@ void srcinst_set_pref_strip(int strip) {
   p->strip = strip;
 }
 
+void srcinst_set_pref_quiet(int quiet) {
+  struct _preferences *p;
+  p = &_srcinst_state.preferences;
+  
+  p->quiet = quiet;
+}
+
 void srcinst_set_pref_install(int install) {
   struct _preferences* p;
   p = &_srcinst_state.preferences;
