@@ -55,4 +55,7 @@ void srcinst_error(int errcode, char* required_string, char* opt_string);
    OPT_STRING is optional. Avoid it passing (char*)0 */
 void srcinst_warning(int warcode, char* required_string, char* opt_string);
 
+/* get the string corresponding to errcode. Do not free or change! */
+char* srcinst_error_string(int errcode);
+
 #endif /* SRCINST_ERROR_H */
