@@ -76,5 +76,20 @@ void srcinst_purge_string_list(struct srcinst_string_list* l, int nonexistant);
 
 char* srcinst_collapse_string_list(struct srcinst_string_list* l, char* sep);
 
+
+/* sort a string list using the compare function passed as argument COMPARE */
+void srcinst_sort_string_list(struct srcinst_string_list* list,
+			      int (*compare)(char**, char**));
+
+/* a search function is not needed.
+   Define your order in the string list, then use srcinst_bsearch.
+   
+   int srcinst_search_string_list(struct srcinst_string_list* list, char* str);
+*/
+
+/* prefix each element of the string list with the argument PREFIX */
+void srcinst_prefix_string_list(struct srcinst_string_list* l, char* prefix);
+
+
 #endif /* SRCINST_STRING_LIST_H */
 
