@@ -137,7 +137,13 @@ int srcinst_file_owner(char* filename, uid_t* usr, gid_t* grp) {
 
 off_t srcinst_file_size(char* filename) {
   struct stat buf;
-  return stat(filename, &buf) != 0 ? -1 : buf.st_size;
+
+  if (stat(filename, &buf) != 0)
+    return (off_t)-1;
+
+  return buf.st_size;
+
+  /*  return stat(filename, &buf) != 0 ? -1 : buf.st_size; */
 }
 
 /* format string may contain rwx */
